#!/usr/bin/python

"""
Test bandwidth (using iperf) on linear networks of varying size,
using both kernel and user datapaths.
Use Dichotomy to evaluate the max bandwidth usable without loosing any packet.

Look to the result in stats.txt file produced which contains 3 cols with:
1: the number of nodes,
2: the bandwidth in Mbps
3: the number of packets exchanged during the exp (10 secondes).

We construct a network of N hosts and N-1 switches, connected as follows:

h1 <-> s1 <-> s2 .. sN-1
       |       |    |
       h2      h3   hN

WARNING: by default, the reference controller only supports 16
switches, so this test WILL NOT WORK unless you have recompiled
your controller to support 100 switches (or more.)

In addition to testing the bandwidth across varying numbers
of switches, this example demonstrates:

- creating a custom topology, LinearTestTopo
- using the ping() and iperf() tests from Mininet()
- testing both the kernel and user switches

"""

from mininet.net import Mininet
from mininet.node import UserSwitch, OVSKernelSwitch
from mininet.topo import Topo
from mininet.log import lg
from mininet.util import irange
from time import sleep
from mininet.link import TCLink
from mininet.node import CPULimitedHost
from mininet.util import dumpNodeConnections
from mininet.log import setLogLevel

import sys
flush = sys.stdout.flush

# Extract the number of packets returned by the execution of udp-perf
def GetLastPPS(fichier):
    fp = open (fichier, "r") 
    for l in fp.readlines ():
        list_values = l.split ()
        bw = list_values[0]
        nodes = list_values[1]
        packets = list_values[2]
    fp.close ()
#    print "Dernier bw=",bw," nodes=",nodes," packets=",packets
    return int (packets)

# Compare the server and client number of packet
def ComparePPS():
    ser=GetLastPPS("server.txt")
    cli=GetLastPPS("client.txt")
    return cli == ser


class LinearTestTopo( Topo ):
    "Topology for a string of N hosts and N-1 switches."

    def __init__( self, N, bw, **params ):

        # Initialize topology
        Topo.__init__( self, **params )

        # Create switches and hosts
        hosts = [ self.addHost( 'h%s' % h )
                  for h in irange( 1, N ) ]
        switches = [ self.addSwitch( 's%s' % s )
                     for s in irange( 1, N - 1 ) ]

        # Wire up switches
        last = None
        for switch in switches:
            if last:
                self.addLink( last, switch, 
                              bw=bw, delay='1ns', loss=0, use_htb=True)
            last = switch

        # Wire up hosts
        self.addLink( hosts[ 0 ], switches[ 0 ], 
                      bw=bw, delay='1ns', loss=0, use_htb=True)
        for host, switch in zip( hosts[ 1: ], switches ):
            self.addLink( host, switch, 
                          bw=bw, delay='1ns', loss=0, use_htb=True)

# use Dichotomy to evaluate the max bandwidth usable without loosing any packet.
def linearBandwidthTest( nodes, dico, band , num , pre ):

    results = {}

#    nodes = switchCount + hostCount
    switchCount = (nodes / 2) - 1
    hostCount = switchCount + 1

    switches = { 'reference user': UserSwitch}
#    switches = { 'Open vSwitch kernel': OVSKernelSwitch }
    datapath = 'reference user'
#    datapath = 'Open vSwitch kernel'
    Switch = UserSwitch

# Variables for dichotomy
    bw = band  # Actual bandwidth
    maxOk = 0  # Maximum bw without loose
    minKo = 0  # Minimum BW with loose
    running = 1  # loop condition
    packets = 0  # last packets received by the server
    newbw = bw   # new bandwidth to test

    while (running):
        topo = LinearTestTopo( hostCount, bw )
        net = Mininet( topo=topo, switch=Switch , link=TCLink)
        net.start()
        n = switchCount
        src, dst = net.hosts[ 0 ], net.hosts[ n ]
        print "testing", src.name,"(",src.IP(),") <->", dst.name,"(",dst.IP(),")"
        print ('sleeping:',str(nodes))
        # sleep in function of the number of the nodes to let the processes to start well...
        sleep (nodes)
        print "start time:",src.cmd('date')
        src.cmd('./udp-perf --duration=10 --bandwidth='+str(bw*1000000)+' --nodes='+str(nodes)+'>s.txt&')
        dst.cmd('./udp-perf --client --bandwidth='+str(bw*1000000)+' --nodes='+str(nodes)+' --duration=10 --host='+src.IP()+' >c.txt &')
        src.cmd('wait %./udp-perf')
        dst.cmd('wait %./udp-perf')
        print "stop time:",src.cmd('date')
        net.stop()
        del net
        del topo
        if (dico):
            if (ComparePPS()):
                if (bw>maxOk):
                    maxOk = bw
                    packets = GetLastPPS("server.txt")
                if (0==minKo):
                    newbw = bw * 2
                else:
                    newbw = (minKo - maxOk) / 2 + maxOk
            else:
                if ((0==minKo)or(bw<minKo)):
                    minKo = bw
                newbw = (minKo - maxOk) / 2 + maxOk
            print "bw:",bw," newbw:",newbw
            if ((maxOk>0)and(abs(newbw-bw)<=pre)):
                running = 0
            if ((maxOk==0)and(bw<pre)):
                running = 0
            bw = newbw            
        else:
            running = 0
            packets = GetLastPPS("server.txt")
        print ('sleeping:',str(5)) # str(nodes))
        sleep (5) #nodes)
        
    print
    # write the result in the stats.txt file.
    if (dico):
        fp=open("stats_mininet_"+str(num)+".txt","a")    
        fp.write(str(nodes)+" "+str(bw)+" "+str(packets)+"\n")
        fp.close ()
    return packets

if __name__ == '__main__':

    lg.setLogLevel( 'info' )
    argc = len(sys.argv)
    dico = (argc>0)and(sys.argv[1]=='dico')
    bw = 1.0
    num = 0
    precision = 1.0
    if (dico):
        nodes = int(sys.argv[2])
    else:
        nodes = int(sys.argv[1])
        bw = float(sys.argv[2])

    num = int(sys.argv[3])        
    precision = float(sys.argv[4]) 
    p  = linearBandwidthTest( nodes , dico , bw , num , precision)
    print "nodes:",nodes," packets: ",p, " dico:", dico


